"""Trimming and upscaling

Usage:
  imtrimup [options] <dyndir>

Arguments:
  <dyndir>  : Input folder containing dynamic scans [default: DirChooser]

Options:
  --glob PATTERN  : File matching pattern [default: *.nii*]
  --scale FACTOR  : Sampling scale factor [default: 2:int]
  --refim PATH  : Reference image [default: FileChooser]
  --fcomment COMMENT  : Prefix to add to outputs
  --no-memlim  : Whether to use more memory (faster)
  --store-img-intrmd  : Whether to write output image files
  --store-img  : Whether to write output image sum file
"""
import logging
from pathlib import Path

from niftypet import nimpa

log = logging.getLogger(__name__)


def run(
    dyndir,
    glob="*.nii*",
    scale=2,
    refim="",
    fcomment="",
    no_memlim=False,
    store_img_intrmd=False,
    store_img=False,
):
    dyndir = Path(dyndir)
    assert dyndir.is_dir()
    # get the file data
    fdyns = list(map(str, dyndir.glob(glob)))
    log.info("found %d files", len(fdyns))
    log.info("sort the dynamic images and output the sorted file names and numpy array")
    imdyn = nimpa.niisort(fdyns)
    log.info("trim & upscale")
    res = nimpa.imtrimup(imdyn["files"], scale=scale, memlim=not no_memlim, refim=refim or "",
                         fcomment=fcomment or "", store_img_intrmd=store_img_intrmd,
                         store_img=store_img)
    log.debug("trimmed and scaled images, shape %r", res['im'].shape)
    return res



import logging
from os import getenv
from pathlib import Path

import pytest

HOME = Path(getenv("DATA_ROOT", "~")).expanduser()


@pytest.fixture(scope="session")
def nvml():
    import pynvml

    try:
        pynvml.nvmlInit()
    except pynvml.NVMLError as exc:
        pytest.skip(str(exc))
    return pynvml


@pytest.fixture(scope="session")
def mMRpars():
    nipet = pytest.importorskip("niftypet.nipet")
    params = nipet.get_mmrparams()
    # params["Cnt"]["VERBOSE"] = True
    params["Cnt"]["LOG"] = logging.INFO
    return params


@pytest.fixture(scope="session")
def datain(mMRpars):
    nipet = pytest.importorskip("niftypet.nipet")
    folder_in = HOME / "Ab_PET_mMR_test"
    if not folder_in.is_dir():
        pytest.skip(f"""Cannot find Ab_PET_mMR_test in
${{DATA_ROOT:-~}} ({HOME}).
""")
    return nipet.classify_input(folder_in, mMRpars)


@pytest.fixture(scope="session")
def dimin():
    trt = HOME / "DPUK" / "TRT"
    if not trt.is_dir():
        pytest.skip(f"""Cannot find DPUK/TRT in
${{DATA_ROOT:-~}} ({HOME}).
""")
    return trt


@pytest.fixture(scope="session")
def fimin(dimin):
    mprage = (dimin / "NEW002_ODE_S02442" / "TP0" /
              "NEW002_PETMR_V1_00015_MR_images_MPRAGE_q-_MPRAGE_20200212145346_15.nii")
    if not mprage.is_file():
        pytest.skip("fimin not found")
    return mprage



"""Convert DICOM folder to a NIfTI file

Usage:
  dcm2nii [options] <dcmpth>

Arguments:
  <dcmpth>  : Input folder containing DICOM files [default: DirChooser]

Options:
  --fcomment COMMENT  : Prefix to add to outputs
  --timestamp  : Whether to include a timestamp in the output filename
"""
import logging
from pathlib import Path

from niftypet import nimpa

log = logging.getLogger(__name__)


def run(dcmpth, fcomment="converted-from-DICOM_", timestamp=True):
    dcmpth = Path(dcmpth)
    assert dcmpth.is_dir()
    log.info("convert")
    res = nimpa.dcm2nii(dcmpth, fprefix=fcomment, timestamp=timestamp)
    log.debug("output file:%s", res)
    return res



from pathlib import Path

import pytest


@pytest.mark.timeout(30 * 60) # 30m
def test_dcm2nii(datain):
    dcmpth = Path(datain['mumapDCM'])
    assert not list(dcmpth.glob('test_dcm2nii_*.nii*'))
    dcm2nii = pytest.importorskip("amypet.dcm2nii")
    dcm2nii.run(datain['mumapDCM'], fcomment="test_dcm2nii_")
    assert list(dcmpth.glob('test_dcm2nii_*.nii*'))
    for f in dcmpth.glob('test_dcm2nii_*.nii*'):
        f.unlink()



from pathlib import Path

import pytest


@pytest.fixture
def dyndir(datain, mMRpars):
    # output path
    opth = str(Path(datain["corepath"]).parent / "amypad" / "dyndir")

    res = Path(opth) / "PET" / "multiple-frames"
    if res.is_dir() and len(list(res.glob('*.nii*'))) > 1:
        return res

    nipet = pytest.importorskip("niftypet.nipet")
    hst = nipet.mmrhist(datain, mMRpars)
    # offset for the time from which meaningful events are detected
    toff = nipet.lm.get_time_offset(hst)
    # dynamic frame timings
    # frmdef = ["def", [4, 15], [8, 30], [9, 60], [2, 180], [8, 300]]
    frmdef = ["def", [4, (hst['t1'] - hst['t0'] - toff) / 4]]
    frm_timings = nipet.lm.dynamic_timings(frmdef, offset=toff)
    # nipet.lm.draw_frames(hst, frm_timings["timings"]) # plot frames
    # hardware mu-map
    muhdct = nipet.hdw_mumap(datain, [1, 2, 4], mMRpars, outpath=opth, use_stored=True)

    # object mu-map with alignment
    mupdct = nipet.align_mumap(
        datain,
        mMRpars,
        outpath=opth,
        store=True,
        use_stored=True,
        hst=hst,
        itr=2,
        petopt="ac",
        fcomment="_mu",
        musrc="pct",
    )
    # object mu-map without alignment--straight from DICOM resampled to PET
    # muodct = nipet.obj_mumap(datain, mMRpars, outpath=opth, store=True)

    nipet.mmrchain(
        datain,
        mMRpars,
        frames=frm_timings["timings"],
        mu_h=muhdct,
        mu_o=mupdct,
        itr=5,
        fwhm=0.0,
        outpath=opth,
        fcomment="_dyn",
        store_img=True,
        store_img_intrmd=True,
    )
    return Path(opth) / "PET" / "multiple-frames"


@pytest.mark.timeout(30 * 60) # 30m
def test_imtrimup(dyndir):
    imtrimup = pytest.importorskip("amypet.imtrimup")
    imtrimup.run(dyndir, glob='*_frm?_t*.nii*')
